/**
 * Plugin for using queue for multiple ajax requests.
 *
 * @autor Pavel Máca
 * @github https://github.com/PavelMaca
 * @license MIT
 * @Gist https://gist.github.com/pavelmaca/942485
 */
(function($) {
    var AjaxQueue = function(options) {
        this.options = options || {};
        var oldComplete = options.complete || function() {};
        var completeCallback = function(XMLHttpRequest, textStatus) {
            (function() {
                oldComplete(XMLHttpRequest, textStatus);
            })();
            $.ajaxQueue.currentRequest = null;
            $.ajaxQueue.startNextRequest();
        };
        this.options.complete = completeCallback;
    };
    AjaxQueue.prototype = {
        options: {},
        perform: function() {
            $.ajax(this.options);
        }
    }
    $.ajaxQueue = {
        queue: [],
        currentRequest: null,
        stopped: false,
        stop: function() {
            $.ajaxQueue.stopped = true;
        },
        run: function() {
            $.ajaxQueue.stopped = false;
            $.ajaxQueue.startNextRequest();
        },
        clear: function() {
            $.ajaxQueue.queue = [];
            $.ajaxQueue.currentRequest = null;
        },
        addRequest: function(options) {
            var request = new AjaxQueue(options);
            $.ajaxQueue.queue.push(request);
            $.ajaxQueue.startNextRequest();
        },
        startNextRequest: function() {
            if ($.ajaxQueue.currentRequest) {
                return false;
            }
            var request = $.ajaxQueue.queue.shift();
            if (request) {
                $.ajaxQueue.currentRequest = request;
                request.perform();
            }
        }
    }
})(jQuery);
(function($) {
    var search_op = [];
    $(document).ready(function() {
        $('.autoload_off_btn').click(function() {
            var yes = confirm('注意！接下來會進行對資料庫的實際操作，無法還原，請確保有備份好資料再進行！');
            if (yes == false) {
                return;
            }
            var self = $(this);
            var data = {
                'action': 'mxp_ajax_set_autoload_no',
                'nonce': MXP.nonce,
                'name': self.data('option_name'),
            };
            $.post(MXP.ajaxurl, data, function(res) {
                if (res.success) {
                    console.log(res.data);
                    self.text('已取消');
                    self.prop('disabled', true);
                }
            });
        });
        $('#go_clean_orphan_postmeta').click(function() {
            var yes = confirm('注意！接下來會進行對資料庫的實際操作，無法還原，請確保有備份好資料再進行！');
            if (yes == false) {
                return;
            }
            var data = {
                'action': 'mxp_ajax_clean_orphan',
                'nonce': MXP.nonce,
                'type': 'post',
            };
            $.post(MXP.ajaxurl, data, function(res) {
                if (res.success) {
                    console.log(res.data);
                    $('#go_clean_orphan_postmeta').text('完成清除！');
                    $('#go_clean_orphan_postmeta').prop('disabled', true);
                }
            });
        });
        $('#go_clean_orphan_commentmeta').click(function() {
            var yes = confirm('注意！接下來會進行對資料庫的實際操作，無法還原，請確保有備份好資料再進行！');
            if (yes == false) {
                return;
            }
            var data = {
                'action': 'mxp_ajax_clean_orphan',
                'nonce': MXP.nonce,
                'type': 'comment',
            };
            $.post(MXP.ajaxurl, data, function(res) {
                if (res.success) {
                    console.log(res.data);
                    $('#go_clean_orphan_commentmeta').text('完成清除！');
                    $('#go_clean_orphan_commentmeta').prop('disabled', true);
                }
            });
        });
        $('#go_clean_options').click(function() {
            var yes = confirm('注意！接下來會進行對資料庫的實際操作，確保有備份好資料再進行！\n此操作針對沒在使用的外掛與主題設定進行清除。\n如果有確認不使用的外掛或主題，請停用並刪除，加速比對清除作業。');
            if (yes == false) {
                return;
            }
            $(this).text('作業中！請耐心等待，勿重新整理此頁面！');
            var step = $('#go_clean_options').data('step');
            $('#go_clean_options').prop('disabled', true);
            if (step == 1) {
                var data = {
                    'action': 'mxp_ajax_db_optimize',
                    'nonce': MXP.nonce,
                    'step': step,
                };
                $.post(MXP.ajaxurl, data, function(res) {
                    if (res.success) {
                        console.log(res.data);
                        var str = '<table><thead><tr><th colspan="3">希望清除的欄位名稱，執行順序 1.刪除 2.強制刪除 </th></tr></thead><tbody>';
                        str += '<tr><td><input type="checkbox" id="del_all" name="del_all" checked><label for="del_all">全部刪除</label>｜<input type="checkbox" id="force_all" name="force_all" ><label for="force_all">全部強制刪除</label></td><td><span id="res_">全域選項設定</span></td></tr>';
                        if (res.data !== undefined && res.data.length > 0) {
                            for (var i = 0; i < res.data.length; i++) {
                                // str += '<span id="res_' + res.data[i].replaceAll('/', '_').replaceAll('.','_') + '">' + res.data[i] + '</span></br>';
                                str += '<tr><td><input type="checkbox" class="del_chk" id="del_' + res.data[i] + '" name="del_' + res.data[i] + '" checked><label for="del_' + res.data[i] + '">刪除欄位</label>｜<input type="checkbox" class="force_chk" id="force_' + res.data[i] + '" name="force_' + res.data[i] + '" ><label for="force_' + res.data[i] + '">強制刪除</label></td><td><span id="res_' + res.data[i] + '">' + res.data[i] + '</span></td></tr>';
                            }
                            str += '</tbody></table>';
                        }
                        if (str != '') {
                            search_op = res.data;
                            $('#go_clean_options_result').html('</br>' + str);
                            $('#del_all').click(function() {
                                var checkBoxes = $("input.del_chk");
                                checkBoxes.prop("checked", !checkBoxes.prop("checked"));
                            });
                            $('#force_all').click(function() {
                                var checkBoxes = $("input.force_chk");
                                checkBoxes.prop("checked", !checkBoxes.prop("checked"));
                            });
                            $('#go_clean_options').data('step', 2);
                            $('#go_clean_options').text('一鍵清除');
                        } else {
                            $('#go_clean_options_result').html('很棒！系統是乾淨的～');
                        }
                    } else {
                        console.log(res.data);
                    }
                    $('#go_clean_options').prop('disabled', false);
                });
            }
            if (step == 2) {
                // for (var i = search_op.length - 1; i >= 0; i--) {
                for (var i = 0; i < search_op.length; i++) {
                    var del = $('#del_' + $.escapeSelector(search_op[i])).is(":checked");
                    var force = $('#force_' + $.escapeSelector(search_op[i])).is(":checked");
                    if (del) {
                        var tmp = [];
                        tmp.push(search_op[i]);
                        var ajax_options = {
                            method: "POST",
                            url: MXP.ajaxurl,
                            cache: false,
                            error: function(res) {
                                console.log('Error', res);
                            },
                            success: function(res) {
                                if (res.success) {
                                    console.log('success', res.data);
                                    if (res.data !== undefined && res.data.length > 0) {
                                        for (var i = 0; i < res.data.length; i++) {
                                            var force = $('#force_' + $.escapeSelector(res.data[i])).is(":checked");
                                            var action = force == true ? '強制' : '嘗試';
                                            $('#res_' + $.escapeSelector(res.data[i])).html('<del>' + res.data[i] + '</del><font color="red"> 》已' + action + '清除！</font>');
                                        }
                                    } else {
                                        var get_slug = this.data.split('=');
                                        var el_id = get_slug[get_slug.length - 1];
                                        $('#res_' + $.escapeSelector(el_id)).html('<del>' + el_id + '</del><font color="blue"> 》使用中的參數，不清除。</font>');
                                    }
                                }
                            },
                            data: {
                                'action': 'mxp_ajax_db_optimize',
                                'nonce': MXP.nonce,
                                'step': step,
                                'force': force == true ? '1' : '0',
                                'search_op': tmp
                            }
                        };
                        $.ajaxQueue.addRequest(ajax_options);
                        $('#res_' + $.escapeSelector(search_op[i])).html(search_op[i] + '<font color="blue"> (<strong>作業中！請耐心等待，勿重新整理此頁面！</strong>)</font>');
                    } else {
                        $('#res_' + $.escapeSelector(search_op[i])).html(search_op[i] + '<font color="orange"> (<strong>本次作業不處理！</strong>)</font>');
                    }
                }
                $.ajaxQueue.run();
                $('#go_clean_options').text('清除中...請勿重新整理頁面。');
                // $('#go_clean_options').prop('disabled', false);
            }
        });
        $('#go_reset_wp').click(function() {
            var yes = confirm('你確定知道自己正在將網站內容全部清除重新建置嗎？');
            if (yes == false) {
                return;
            }
            $('#go_reset_wp').text('網站重置中... 請勿離開此畫面，稍後片刻！');
            $('#go_reset_wp').prop('disabled', true);
            var del_uploads = confirm('是否清除上傳目錄資料夾？');
            var clean_all_tables = confirm('是否清除其他非 WordPress 內建資料表？');
            var keep_options_reset = confirm('是否保留網站系統設定？');
            var password = prompt("請輸入新的管理員密碼。（留空預設為使用當前使用者密碼）", "");
            var domain = prompt("最後確認，請輸入本網站網域名稱 " + window.location.hostname, "");
            if (domain != window.location.hostname) {
                alert('比對錯誤，中斷執行。');
                $('#go_reset_wp').prop('disabled', false);
                return;
            }
            var data = {
                'action': 'mxp_ajax_reset_wp',
                'nonce': MXP.nonce,
                'del_uploads': del_uploads == true ? '1' : '0',
                'clean_all_tables': clean_all_tables == true ? '1' : '0',
                'keep_options_reset': keep_options_reset == true ? '1' : '0',
                'password': password
            };
            $.post(MXP.ajaxurl, data, function(res) {
                if (res.success) {
                    console.log(res.data);
                    if (res.data.password != '') {
                        var newpassword = prompt("重置網站成功！新密碼如下，複製與確認後將會跳轉登入", res.data.password);
                    } else {
                        alert('重置網站成功！接下來會跳轉登入頁面，請使用原密碼登入網站。');
                    }
                    location.href = res.data.url + '/wp-admin/';
                } else {
                    console.log(res.data);
                    alert('重置網站失敗！')
                }
                $('#go_reset_wp').prop('disabled', false);
            });
        });
    });
})(jQuery);