<?php
/**
 * Plugin Name: Dev Tools - Mxp.TW
 * Plugin URI: https://goo.gl/2gLq18
 * Description: 一介資男の常用外掛整理與常用開發功能整合外掛。
 * Version: 2.3.8
 * Author: Chun
 * Author URI: https://www.mxp.tw/contact/
 * License: MIT
 */

if (!defined('WPINC')) {
    die;
}
class MxpDevTools {
    static $VERSION                   = '2.3.8';
    private $themeforest_api_base_url = 'https://api.envato.com/v3';
    protected static $instance        = null;
    public $plugin_slug               = 'mxp_wp_dev_tools';
    private $installed_plugins        = null;
    private function __construct() {
        $this->init();
    }
    public function init() {
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'add_action_links'));
        add_filter('plugin_action_links', array($this, 'mxp_add_plugin_download_link'), 11, 4);
        add_action('admin_init', array($this, 'mxp_init_author_plugins_table'));
        add_action('admin_enqueue_scripts', array($this, 'load_assets'));
        add_action('admin_menu', array($this, 'create_plugin_menu'));
        add_action('wp_ajax_mxp_install_plugin', array($this, 'mxp_ajax_install_plugin'));
        add_action('wp_ajax_mxp_install_plugin_from_url', array($this, 'mxp_ajax_install_plugin_from_url'));
        add_action('wp_ajax_mxp_activate_plugin', array($this, 'mxp_ajax_activate_plugin'));
        add_action('wp_ajax_mxp_ajax_mysqldump', array($this, 'mxp_ajax_mysqldump'));
        add_action('wp_ajax_mxp_current_plugin_download', array($this, 'mxp_ajax_current_plugin_download_action'));
        add_action('wp_ajax_mxp_install_theme', array($this, 'mxp_ajax_install_theme'));
        add_action('wp_ajax_mxp_ajax_search_replace_db', array($this, 'mxp_ajax_search_replace_db'));
        add_action('wp_ajax_mxp_ajax_db_optimize', array($this, 'mxp_ajax_db_optimize'));
        add_action('wp_ajax_mxp_ajax_clean_orphan', array($this, 'mxp_ajax_clean_orphan'));
        add_action('wp_ajax_mxp_ajax_set_autoload_no', array($this, 'mxp_ajax_set_autoload_no'));
        add_action('wp_ajax_mxp_ajax_reset_wp', array($this, 'mxp_ajax_reset_wp'));
        wp_cache_flush();
    }
    public static function get_instance() {
        if (!isset(self::$instance) && is_super_admin()) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    public function load_assets() {
        wp_register_script($this->plugin_slug . '-plugins-list', plugin_dir_url(__FILE__) . 'includes/assets/js/plugins-list/app.js', array('jquery'), self::$VERSION, false);
        wp_register_script($this->plugin_slug . '-themeforest-list', plugin_dir_url(__FILE__) . 'includes/assets/js/themeforest-list/app.js', array('jquery'), self::$VERSION, false);
        wp_register_script($this->plugin_slug . '-search-plugins', plugin_dir_url(__FILE__) . 'includes/assets/js/search-plugins/app.js', array('jquery'), self::$VERSION, false);
        wp_register_script($this->plugin_slug . '-db-search-replace', plugin_dir_url(__FILE__) . 'includes/assets/js/search-replace/app.js', array('jquery'), self::$VERSION, false);
        wp_register_script($this->plugin_slug . '-db-optimize', plugin_dir_url(__FILE__) . 'includes/assets/js/db-optimize/app.js', array('jquery'), self::$VERSION, false);
        wp_register_script($this->plugin_slug . '-dashboard', plugin_dir_url(__FILE__) . 'includes/assets/js/dashboard/app.js', array('jquery'), self::$VERSION, false);
    }

    public function create_plugin_menu() {
        add_menu_page('Mxp.TW 開發常用工具箱', '開發工具箱', 'administrator', $this->plugin_slug, array($this, 'main_page_cb'), 'dashicons-admin-generic');
        add_submenu_page($this->plugin_slug, 'Themeforest List', 'Themeforest List', 'administrator', 'mxp-themeforest-list', array($this, 'themeforest_page_cb'));
        add_submenu_page($this->plugin_slug, '調整內容作者工具', '調整內容作者工具', 'administrator', 'mxp-change-post-owner', array($this, 'changepostowner_page_cb'));
        add_submenu_page($this->plugin_slug, '作者外掛列表', '作者外掛列表', 'administrator', 'mxp-author-plugins', array($this, 'listauthorplugin_page_cb'));
        add_submenu_page($this->plugin_slug, '外掛搜尋工具', '外掛搜尋工具', 'administrator', 'mxp-search-plugins', array($this, 'searchplugin_page_cb'));
        add_submenu_page($this->plugin_slug, '主題打包工具', '主題打包工具', 'administrator', 'mxp-theme-archive', array($this, 'themearchive_page_cb'));
        add_submenu_page($this->plugin_slug, '查看網站各項設定', '查看網站各項設定', 'administrator', 'mxp-get-wp-config', array($this, 'getwpconfig_page_cb'));
        add_submenu_page($this->plugin_slug, '資料庫檢視與匯出', '資料庫檢視與匯出', 'administrator', 'mxp-db-op-methods', array($this, 'dbopmethods_page_cb'));
        add_submenu_page($this->plugin_slug, '資料庫關鍵字取代', '資料庫關鍵字取代', 'administrator', 'mxp-db-replace-methods', array($this, 'dbreplacemethods_page_cb'));
        add_submenu_page($this->plugin_slug, 'WP 資料庫最佳化', 'WP 資料庫最佳化', 'administrator', 'mxp-wpdb-optimize-methods', array($this, 'wpdboptimizemethods_page_cb'));
        add_submenu_page($this->plugin_slug, '查看最近變更檔案', '查看最近變更檔案', 'administrator', 'mxp-recently-mod_files', array($this, 'recentlymodfiles_page_cb'));
    }

    public function add_action_links($links) {
        $mxp_links = array(
            '<a href="' . admin_url('admin.php?page=mxp_wp_dev_tools') . '"><font color=red>點此設定</font></a>',
        );
        return array_merge($links, $mxp_links);
    }

    public function mxp_add_plugin_download_link($actions, $plugin_file, $plugin_data, $context) {
        if (!is_super_admin()) {
            return $actions;
        }
        $new_actions = array();
        if (strpos($plugin_file, '/') !== false) {
            $type = 'folder';
        } else {
            $type = 'file';
        }
        $path = '';
        switch ($context) {
        case 'mustuse':
            $path = WPMU_PLUGIN_DIR . '/' . $plugin_file;
            break;
        case 'dropins':
            $path = WP_CONTENT_DIR . '/' . $plugin_file;
            break;
        case 'all':
        case 'active':
        case 'active':
        case 'inactive':
        case 'recently_activated':
        case 'auto-update-disabled':
        case 'upgrade':
        default:
            $path = WP_PLUGIN_DIR . '/' . $plugin_file;
            break;
        }
        if ($path != '') {
            $mxp_download_action_link = admin_url('admin-ajax.php?action=mxp_current_plugin_download&path=' . base64_encode($path) . '&type=' . $type . '&context=' . $context);
            $mxp_download_action_link = add_query_arg('_wpnonce', wp_create_nonce('mxp-download-current-plugins-' . base64_encode($path)), $mxp_download_action_link);
            $download_link            = '<a target="_blank" href="' . esc_url($mxp_download_action_link) . '" class="mxp_plugin_download_link">打包外掛</a>';

            $new_actions['mxp-donwload-current-plugin'] = $download_link;
        }
        return array_merge($new_actions, $actions);
    }

    public function get_recently_mod_files($day_from = '', $day_to = '') {
        $recently_mod_files = array(
            'dot_files'           => array(),
            'core_files'          => array(),
            'uploads_files'       => array(),
            'uncategorized_files' => array(),
            'theme_files'         => array(),
            'plugin_files'        => array(),
        );
        $day_from = strtotime($day_from);
        $day_to   = strtotime($day_to);
        if ($day_from == '' || $day_to == '' || $day_from == false || $day_to == false) {
            return $recently_mod_files;
        }
        // 如果前後時間選錯，直接排序過
        if ($day_to < $day_from) {
            $tmp      = $day_to;
            $day_to   = $day_from;
            $day_from = $tmp;
        }
        // 如果選到同一天，就抓前一天到當天
        if ($day_to == $day_from) {
            $day_from -= DAY_IN_SECONDS;
        }
        // 結束日抓完整的一日來看
        $day_to += DAY_IN_SECONDS;
        $theme_root      = get_theme_root();
        $plugin_root     = WP_PLUGIN_DIR;
        $uploads_root    = wp_get_upload_dir()['basedir'];
        $all_files_in_WP = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(ABSPATH, RecursiveDirectoryIterator::SKIP_DOTS));
        foreach ($all_files_in_WP as $key => $file) {
            $entry = $file->getPathname();
            if ($file->isDir()) {
                continue;
            }
            $path_parts = pathinfo($entry);
            //先記錄所有隱藏檔案
            if (strpos($path_parts['basename'], '.') === 0) {
                if ('.DS_Store' === $path_parts['basename']) {
                    @unlink($entry);
                    continue;
                }
                $recently_mod_files['dot_files'][] = array('full_path' => $entry, 'relative_path' => str_replace(ABSPATH, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                continue;
            }
            // 判斷是否是核心檔案， WP_CONTENT_DIR 之外的都算
            if (strpos($entry, WP_CONTENT_DIR) === 0) {
                // 一天內修改的檔案
                if ($file->getMTime() >= $day_from && $file->getMTime() <= $day_to) {
                    // 非系統檔案
                    if (strpos($entry, $theme_root) === 0) {
                        // 主題檔案
                        $recently_mod_files['theme_files'][] = array('full_path' => $entry, 'relative_path' => str_replace($theme_root . DIRECTORY_SEPARATOR, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                        continue;
                    }
                    if (strpos($entry, $plugin_root) === 0) {
                        // 外掛檔案
                        $recently_mod_files['plugin_files'][] = array('full_path' => $entry, 'relative_path' => str_replace($plugin_root . DIRECTORY_SEPARATOR, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                        continue;
                    }
                    if (strpos($entry, $uploads_root) === 0) {
                        // 上傳檔案
                        $recently_mod_files['uploads_files'][] = array('full_path' => $entry, 'relative_path' => str_replace($uploads_root . DIRECTORY_SEPARATOR, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                        continue;
                    }
                    // 其他未分類檔案
                    $recently_mod_files['uncategorized_files'][] = array('full_path' => $entry, 'relative_path' => str_replace(WP_CONTENT_DIR . DIRECTORY_SEPARATOR, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                }
            } else {
                // 算系統檔案（預設紀錄一天內修改的檔案）
                if ($file->getMTime() >= $day_from && $file->getMTime() <= $day_to) {
                    $recently_mod_files['core_files'][] = array('full_path' => $entry, 'relative_path' => str_replace(ABSPATH, '', $entry), 'name' => $path_parts['basename'], 'mod_time' => $file->getMTime());
                    continue;
                }
            }
        }
        return $recently_mod_files;
    }

    public function mxp_ajax_reset_wp() {
        if (!isset($_REQUEST['nonce']) || !wp_verify_nonce($_REQUEST['nonce'], 'mxp-ajax-nonce-for-db-optimize') || !is_super_admin()) {
            wp_send_json_error('請求驗證有誤！');
        }
        $del_uploads        = false;
        $clean_all_tables   = true;
        $keep_options_reset = true;
        $password           = '';
        if (isset($_REQUEST['del_uploads']) && $_REQUEST['del_uploads'] != '') {
            $del_uploads = sanitize_text_field($_REQUEST['del_uploads']);
            if ($del_uploads != '0') {
                $del_uploads = true;
            }
        }
        if (isset($_REQUEST['clean_all_tables']) && $_REQUEST['clean_all_tables'] != '') {
            $clean_all_tables = sanitize_text_field($_REQUEST['clean_all_tables']);
            if ($clean_all_tables != '1') {
                $clean_all_tables = false;
            }
        }
        if (isset($_REQUEST['keep_options_reset']) && $_REQUEST['keep_options_reset'] != '') {
            $keep_options_reset = sanitize_text_field($_REQUEST['keep_options_reset']);
            if ($keep_options_reset != '1') {
                $keep_options_reset = false;
            }
        }
        if (isset($_REQUEST['password']) && $_REQUEST['password'] != '') {
            $password = sanitize_text_field($_REQUEST['password']);
        }
        if ($del_uploads) {
            require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-base.php';
            require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-direct.php';
            $fileSystemDirect = new WP_Filesystem_Direct(false);
            $upload_dir       = wp_upload_dir();
            $fileSystemDirect->rmdir($upload_dir['basedir'], true);
        }
        require_once ABSPATH . '/wp-admin/includes/upgrade.php';
        global $wpdb;
        $oldsite_options = array();
        // 定義在 wp-admin/includes/schema.php
        $keep_options = array('siteurl', 'home', 'blogname', 'blogdescription', 'users_can_register', 'admin_email', 'start_of_week', 'use_balanceTags', 'use_smilies', 'require_name_email', 'comments_notify', 'posts_per_rss', 'rss_use_excerpt', 'mailserver_url', 'mailserver_login', 'mailserver_pass', 'mailserver_port', 'default_category', 'default_comment_status', 'default_ping_status', 'default_pingback_flag', 'posts_per_page', 'date_format', 'time_format', 'links_updated_date_format', 'comment_moderation', 'moderation_notify', 'permalink_structure', 'rewrite_rules', 'hack_file', 'blog_charset', 'moderation_keys', 'category_base', 'ping_sites', 'comment_max_links', 'gmt_offset', 'default_email_category', 'recently_edited', 'template', 'stylesheet', 'comment_registration', 'html_type', 'use_trackback', 'default_role', 'db_version', 'uploads_use_yearmonth_folders', 'upload_path', 'blog_public', 'default_link_category', 'show_on_front', 'tag_base', 'show_avatars', 'avatar_rating', 'upload_url_path', 'thumbnail_size_w', 'thumbnail_size_h', 'thumbnail_crop', 'medium_size_w', 'medium_size_h', 'avatar_default', 'large_size_w', 'large_size_h', 'image_default_link_type', 'image_default_size', 'image_default_align', 'close_comments_for_old_posts', 'close_comments_days_old', 'thread_comments', 'thread_comments_depth', 'page_comments', 'comments_per_page', 'default_comments_page', 'comment_order', 'sticky_posts', 'widget_categories', 'widget_text', 'widget_rss', 'timezone_string', 'page_for_posts', 'page_on_front', 'default_post_format', 'link_manager_enabled', 'finished_splitting_shared_terms', 'site_icon', 'medium_large_size_w', 'medium_large_size_h', 'wp_page_for_privacy_policy', 'show_comments_cookies_opt_in', 'admin_email_lifespan', 'disallowed_keys', 'comment_previously_approved', 'auto_plugin_theme_update_emails', 'auto_update_core_dev', 'auto_update_core_minor', 'auto_update_core_major', 'wp_force_deactivated_plugins');
        foreach ($keep_options as $index => $ops_key) {
            $oldsite_options[$ops_key] = get_option($ops_key);
        }
        $blogname    = $oldsite_options['blogname'];
        $admin_email = $oldsite_options['admin_email'];
        $blog_public = $oldsite_options['blog_public'];
        $languages   = get_locale();
        $user        = wp_get_current_user();
        // 找到所有資料表
        $tables            = $wpdb->get_col("SHOW TABLES");
        $wp_default_tables = array_values($wpdb->tables());
        foreach ($tables as $table) {
            if (in_array($table, $wp_default_tables)) {
                // 內建資料表全給他砍了～
                $wpdb->query("DROP TABLE `$table`");
            } else {
                // 保留彈性不一定要砍掉其他資料表
                if ($clean_all_tables) {
                    $wpdb->query("DROP TABLE `$table`");
                }
            }
        }
        // 如果沒有要保留先前設定，就判斷預設主題是否有安裝，沒有就安裝上。
        if (!$keep_options_reset) {
            $theme = wp_get_theme(WP_DEFAULT_THEME);
            if (!$theme->exists()) {
                require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
                include_once ABSPATH . 'wp-admin/includes/theme.php';
                $api = themes_api(
                    'theme_information',
                    array(
                        'slug'   => WP_DEFAULT_THEME,
                        'fields' => array('sections' => false),
                    )
                );
                if (!is_wp_error($api)) {
                    $skin     = new WP_Ajax_Upgrader_Skin();
                    $upgrader = new Theme_Upgrader($skin);
                    $result   = $upgrader->install($api->download_link);
                }
            }
        }
        // 重新安裝 WordPress
        $result = wp_install($blogname, $user->user_login, $user->user_email, $blog_public, '', $password, $languages);
        if ($password == '') {
            $user_id = $result['user_id'];
            // 沒指定新密碼的話，就更新原本的密碼
            $query = $wpdb->prepare("UPDATE $wpdb->users SET user_pass = %s, user_activation_key = '' WHERE ID = %d", $user->user_pass, $user_id);
            $wpdb->query($query);
            update_user_meta($user_id, 'default_password_nag', false);
            update_user_meta($user_id, $wpdb->prefix . 'default_password_nag', false);
            $result['password'] = $password; //空的
        }
        // 重新啟用工具箱外掛
        @activate_plugin(plugin_basename(__FILE__));
        // 重新把設定存回去
        foreach ($oldsite_options as $option => $value) {
            if ($keep_options_reset) {
                update_option($option, $value);
            }
        }
        wp_send_json_success($result);
    }

    public function mxp_ajax_set_autoload_no() {
        if (!isset($_REQUEST['name']) || $_REQUEST['name'] == '') {
            wp_send_json_error('請求參數有誤！');
        }
        $name = sanitize_text_field($_REQUEST['name']);
        if (!isset($_REQUEST['nonce']) || !wp_verify_nonce($_REQUEST['nonce'], 'mxp-ajax-nonce-for-db-optimize')) {
            wp_send_json_error('請求驗證有誤！');
        }
        global $wpdb;
        $res = $wpdb->get_results(
            $wpdb->prepare("UPDATE {$wpdb->options} SET autoload=%s WHERE option_name=%s", 'no', $name), ARRAY_A
        );
        wp_send_json_success($res);
    }
    public function mxp_ajax_clean_orphan() {
        if (!isset($_REQUEST['type']) || $_REQUEST['type'] == '') {
            wp_send_json_error('請求參數有誤！');
        }
        $type = sanitize_text_field($_REQUEST['type']);
        if (!isset($_REQUEST['nonce']) || !wp_verify_nonce($_REQUEST['nonce'], 'mxp-ajax-nonce-for-db-optimize')) {
            wp_send_json_error('請求驗證有誤！');
        }
        global $wpdb;
        $res = '';
        if ($type == 'post') {
            $res = $wpdb->get_results("DELETE pm FROM {$wpdb->postmeta} pm LEFT JOIN {$wpdb->posts} wp ON wp.ID = pm.post_id WHERE wp.ID IS NULL");
        }
        if ($type == 'comment') {
            $res = $wpdb->get_results("DELETE FROM {$wpdb->commentmeta} WHERE comment_id NOT IN (SELECT comment_id FROM {$wpdb->comments})");
        }
        wp_send_json_success($res);
    }
    public function mxp_ajax_db_optimize() {
        set_time_limit(0);
        if (!isset($_REQUEST['step']) || (intval($_REQUEST['step']) != 1 && intval($_REQUEST['step']) != 2)) {
            wp_send_json_error('請求參數有誤！');
        }
        $step      = sanitize_text_field($_REQUEST['step']);
        $force     = false;
        $search_op = array();
        if ($step == 2 && isset($_REQUEST['search_op']) && is_array($_REQUEST['search_op'])) {
            $search_op = array_map('sanitize_text_field', $_REQUEST['search_op']);
            $force     = sanitize_text_field($_REQUEST['force']);
            if ($force != '0') {
                $force = true;
            }
        }
        if (!isset($_REQUEST['nonce']) || !wp_verify_nonce($_REQUEST['nonce'], 'mxp-ajax-nonce-for-db-optimize')) {
            wp_send_json_error('請求驗證有誤！');
        }
        global $wpdb;
        if ($step == 1) {
            //預設的設定資料欄位不清除
            $default_option_name = array('active_plugins', 'admin_email', 'admin_email_lifespan', 'auto_plugin_theme_update_emails', 'auto_update_core_dev', 'auto_update_core_major', 'auto_update_core_minor', 'avatar_default', 'avatar_rating', 'blog_charset', 'blog_public', 'blogdescription', 'blogname', 'category_base', 'close_comments_days_old', 'close_comments_for_old_posts', 'comment_max_links', 'comment_moderation', 'comment_order', 'comment_previously_approved', 'comment_registration', 'comments_notify', 'comments_per_page', 'cron', 'date_format', 'db_version', 'default_category', 'default_comment_status', 'default_comments_page', 'default_email_category', 'default_link_category', 'default_ping_status', 'default_pingback_flag', 'default_post_format', 'default_role', 'disallowed_keys', 'finished_splitting_shared_terms', 'fresh_site', 'gmt_offset', 'hack_file', 'home', 'html_type', 'https_detection_errors', 'image_default_align', 'image_default_link_type', 'image_default_size', 'initial_db_version', 'large_size_h', 'large_size_w', 'link_manager_enabled', 'links_updated_date_format', 'mailserver_login', 'mailserver_pass', 'mailserver_port', 'mailserver_url', 'medium_large_size_h', 'medium_large_size_w', 'medium_size_h', 'medium_size_w', 'moderation_keys', 'moderation_notify', 'page_comments', 'page_for_posts', 'page_on_front', 'permalink_structure', 'ping_sites', 'posts_per_page', 'posts_per_rss', 'recently_edited', 'recovery_keys', 'require_name_email', 'rewrite_rules', 'rss_use_excerpt', 'show_avatars', 'show_comments_cookies_opt_in', 'show_on_front', 'sidebars_widgets', 'site_icon', 'siteurl', 'start_of_week', 'sticky_posts', 'stylesheet', 'tag_base', 'template', 'thread_comments', 'thread_comments_depth', 'thumbnail_crop', 'thumbnail_size_h', 'thumbnail_size_w', 'time_format', 'timezone_string', 'uninstall_plugins', 'upload_path', 'upload_url_path', 'uploads_use_yearmonth_folders', 'use_balanceTags', 'use_smilies', 'use_trackback', 'user_count', 'users_can_register', 'wp_force_deactivated_plugins', 'wp_page_for_privacy_policy', 'WPLANG');
            // 撈出全部欄位
            $all_option_name = $wpdb->get_results("SELECT option_id,option_name FROM {$wpdb->options}", ARRAY_A);
            // 撈出空值欄位
            $empty_value_options = $wpdb->get_results("SELECT * FROM {$wpdb->options} WHERE option_value IS NULL OR option_value =''", ARRAY_A);
            // 最後比對的欄位組
            $search_op            = array();
            $current_theme        = get_option('stylesheet');
            $current_parent_theme = get_option('template');
            // 最後刪除的欄位
            $deleted_options = array();
            foreach ($empty_value_options as $key => $ops) {
                if (in_array($ops['option_name'], $default_option_name, true)) {
                    continue;
                }
                $search_op[$ops['option_name']] = $ops['option_id'];
            }
            foreach ($all_option_name as $key => $ops) {
                if (in_array($ops['option_name'], $default_option_name, true)) {
                    continue;
                }
                if (strpos($ops['option_name'], '_site_transient_timeout_') === 0) {
                    $del_transient     = str_replace('_timeout', '', $ops['option_name']);
                    $deleted_options[] = $del_transient;
                    $deleted_options[] = $ops['option_name'];
                    delete_option($del_transient);
                    delete_option($ops['option_name']);
                }
                if (strpos($ops['option_name'], '_transient_timeout_') === 0) {
                    $del_transient     = str_replace('_timeout', '', $ops['option_name']);
                    $deleted_options[] = $del_transient;
                    $deleted_options[] = $ops['option_name'];
                    delete_option($del_transient);
                    delete_option($ops['option_name']);
                }
                if (strpos($ops['option_name'], '_site_transient') === 0) {
                    continue;
                }
                if (strpos($ops['option_name'], '_transient') === 0) {
                    continue;
                }
                if (strpos($ops['option_name'], '_user_roles') !== false) {
                    continue;
                }
                if (strpos($ops['option_name'], 'widget_') === 0) {
                    continue;
                }
                $search_op[$ops['option_name']] = $ops['option_id'];
            }
            // 當前主題的設定不清除
            if (isset($search_op['theme_mods_' . $current_theme])) {
                unset($search_op['theme_mods_' . $current_theme]);
            }
            if (isset($search_op['theme_mods_' . $current_parent_theme])) {
                unset($search_op['theme_mods_' . $current_parent_theme]);
            }
            wp_send_json_success(array_keys($search_op));
        }
        if ($step == 2) {
            $deleted_options = array();
            if ($force) {
                foreach ($search_op as $option_name) {
                    $deleted_options[] = $option_name;
                    $res               = $wpdb->delete(
                        $wpdb->prefix . 'options',
                        array('option_name' => $option_name),
                        array('%s')
                    );
                    if ($wpdb->last_error !== '') {
                        error_log('last_query: ' . $wpdb->last_query . ' & last_error: ' . $wpdb->last_error);
                        wp_send_json_error(array('last_query' => $wpdb->last_query, 'last_error' => $wpdb->last_error));
                    }
                }
                wp_send_json_success($deleted_options);
            }
            foreach ($search_op as $option_name) {
                $in_use_flag     = false;
                $all_files_in_WP = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(ABSPATH, RecursiveDirectoryIterator::SKIP_DOTS));
                foreach ($all_files_in_WP as $key => $file) {
                    $entry = $file->getPathname();
                    if ($file->isDir()) {
                        continue;
                    }
                    $path_parts = pathinfo($entry);
                    if (isset($path_parts['extension']) && strtoupper($path_parts['extension']) == 'PHP' && $entry != __FILE__) {
                        if (strpos(file_get_contents($entry), $option_name) !== false) {
                            $in_use_flag = true;
                        }
                    }
                }
                if ($in_use_flag != true) {
                    // print_r($option_name . ' => ID: MXP' . PHP_EOL);
                    $deleted_options[] = $option_name;
                    delete_option($option_name);
                }
            }
            wp_send_json_success($deleted_options);
        }
    }
    public function mxp_ajax_search_replace_db() {
        set_time_limit(0);
        ini_set("memory_limit", "1024MB");
        $dbname       = '';
        $dry_run      = false;
        $tables       = array();
        $nonce        = '';
        $replace_from = array();
        $replace_to   = array();

        if (!isset($_POST['dbname']) || $_POST['dbname'] == '' || !isset($_POST['tables']) || !is_array($_POST['tables']) ||
            !isset($_POST['replace_from']) || !is_array($_POST['replace_from']) ||
            !isset($_POST['replace_to']) || !is_array($_POST['replace_to'])) {
            wp_send_json_error('請求參數有誤！');
        }
        $dbname = sanitize_text_field($_POST['dbname']);
        if (isset($_POST['dry_run']) && $_POST['dry_run'] != '') {
            $dry_run = true;
        }
        $tables       = array_map('sanitize_text_field', $_POST['tables']);
        $replace_from = array_map('sanitize_text_field', $_POST['replace_from']);
        $replace_to   = array_map('sanitize_text_field', $_POST['replace_to']);
        if (!isset($_REQUEST['nonce']) || !wp_verify_nonce($_REQUEST['nonce'], 'mxp-ajax-nonce-for-db-search-replace-' . $dbname)) {
            wp_send_json_error('請求驗證有誤！');
        }
        include dirname(__FILE__) . '/mxp_srdb.class.php';
        $args = array(
            'dbname'         => $dbname,
            'search'         => $replace_from,
            'replace'        => $replace_to,
            'tables'         => $tables,
            'exclude_tables' => array(),
            'exclude_cols'   => array(),
            'include_cols'   => array(),
            'dry_run'        => $dry_run,
            'regex'          => false,
            'page_size'      => 500,
            'verbose'        => false,
            'debug'          => false,
        );
        $sr  = new Mxp_SRDB($args);
        $res = array('report' => $sr->report, 'errors' => $sr->errors);
        wp_send_json_success($res);
    }

    public function mxp_ajax_mysqldump() {
        set_time_limit(0);
        ini_set("memory_limit", "1024MB");
        if (!isset($_REQUEST['database']) || $_REQUEST['database'] == '') {
            die('沒指定匯出資料庫！');
        }
        if (!isset($_REQUEST['table']) || $_REQUEST['table'] == '') {
            die('沒指定匯出資料表！');
        }
        if (!isset($_REQUEST['_wpnonce']) || !wp_verify_nonce($_REQUEST['_wpnonce'], 'mxp-mysqldump-' . $_REQUEST['database'] . '-' . $_REQUEST['table'])) {
            die('請求驗證有誤！');
        }
        $export_database = sanitize_text_field($_REQUEST['database']);
        $export_table    = sanitize_text_field($_REQUEST['table']);
        global $wpdb;
        $sql_name      = $export_database . '-' . $export_table . '-' . date('Y-m-d-H-i-s') . '.sql';
        $sql_full_path = sys_get_temp_dir() . '/' . $sql_name;
        $zip_file_name = $sql_name . '.zip';
        $zip_file_path = sys_get_temp_dir() . '/' . $zip_file_name;
        // error_log($sql_full_path);
        $fp = fopen($sql_full_path, 'w');
        fwrite($fp, '-- MXP MySQL Dump' . PHP_EOL);
        fwrite($fp, '-- version 1.0' . PHP_EOL);
        fwrite($fp, '-- https://www.mxp.tw/' . PHP_EOL);
        fwrite($fp, '-- 產生時間： ' . date('Y-m-d H:i:s') . PHP_EOL);
        fwrite($fp, '-- 伺服器版本： ' . $wpdb->db_version() . PHP_EOL);
        fwrite($fp, '-- PHP 版本： ' . phpversion() . PHP_EOL . PHP_EOL);
        fwrite($fp, 'SET SQL_MODE  = "NO_AUTO_VALUE_ON_ZERO";' . PHP_EOL);
        fwrite($fp, 'SET foreign_key_checks = 0;' . PHP_EOL);
        // fwrite($fp, 'SET sql_big_selects = 1;' . PHP_EOL);
        fwrite($fp, 'SET time_zone = "+00:00";' . PHP_EOL);
        fwrite($fp, '/*!40101 SET NAMES utf8mb4 */;' . PHP_EOL . PHP_EOL);
        $tables      = $wpdb->get_results("SHOW FULL TABLES FROM {$export_database}", ARRAY_A);
        $dump_tables = array();
        foreach ($tables as $tables_index => $_table) {
            $dump_tables[$_table['Tables_in_' . $export_database]] = $_table['Table_type'];
        }
        $tables_info = array();
        foreach ($dump_tables as $table_name => $table_type) {
            $results                  = $wpdb->get_results("DESC `{$export_database}`.`{$table_name}`");
            $tables_info[$table_name] = array();
            foreach ($results as $row) {
                // print_r("Field: " . $row->Field . PHP_EOL);
                // print_r("Type: " . $row->Type . PHP_EOL);
                $tables_info[$table_name][/*$row->Field*/] = strtoupper(current(explode('(', $row->Type)));
            }
        }
        $login_user = $wpdb->get_results("SELECT USER()", ARRAY_A);
        if ($export_table != "ALL") {
            $new_dump_tables = array();
            if (isset($dump_tables[$export_table])) {
                $new_dump_tables[$export_table] = $dump_tables[$export_table];
            }
            $dump_tables = $new_dump_tables;
        }
        foreach ($dump_tables as $table => $table_type) {
            if (strpos($table_type, 'TABLE') !== false) {
                $table_type = 'TABLE';
            }
            $schema     = $wpdb->get_results("SHOW CREATE {$table_type} `{$export_database}`.`{$table}`;", ARRAY_A);
            $schema_str = '';
            foreach ($schema[0] as $key => $query) {
                if (strpos($key, 'Create') === 0) {
                    $schema_str = $query;
                    if (in_array(strtoupper($table_type), array('VIEW', 'EVENTS', 'TRIGGERS', 'PROCEDURE', 'FUNCTION'))) {
                        $split      = explode(' ', $query);
                        $schema_str = '';
                        foreach ($split as $split_index => $query_part) {
                            if (strpos($query_part, 'DEFINER=') !== 0) {
                                $schema_str .= $query_part . ' ';
                            }
                        }
                        // $schema_str = preg_replace('~^([A-Z =]+) DEFINER=`' . preg_replace('~@(.*)~', '`@`(%|\1)', '.*@' . $wpdb->dbhost) . '`~', '\1', $query);
                    }
                }
            }
            fwrite($fp, "DROP {$table_type} IF EXISTS {$table};" . PHP_EOL . PHP_EOL);
            fwrite($fp, $schema_str . ";" . PHP_EOL . PHP_EOL);
            $offset = 0;
            while (true && $table_type == 'TABLE') {
                $inserts = $wpdb->get_results(
                    $wpdb->prepare("SELECT * FROM `$export_database`.`{$table}` LIMIT %d, 50;", $offset)
                    , ARRAY_A);
                $count       = count($inserts);
                $fields_type = $tables_info[$table];
                foreach ($inserts as $index => $row) {
                    // $fields   = array_map(array($wpdb, '_real_escape'), array_keys($row));
                    // $data_set = array_map(array($wpdb, '_real_escape'), array_values($row));
                    $fields   = array_keys($row);
                    $data_set = array();
                    if ((strpos($table, "\r") !== false) || (strpos($table, "\n") !== false)) {
                        $table = str_replace(["\r", "\n"], ['\\r', '\\n'], $table);
                    }
                    $table = str_replace("`", "``", $table);
                    $query = "INSERT INTO `{$table}` (";
                    foreach ($fields as $field_index => $field) {
                        if ((strpos($field, "\r") !== false) || (strpos($field, "\n") !== false)) {
                            $field = str_replace(["\r", "\n"], ['\\r', '\\n'], $field);
                        }
                        $field = str_replace("`", "``", $field);
                        $query .= "`{$field}`";
                        if (count($fields) > $field_index + 1) {
                            $query .= ", ";
                        }
                        $data_set[] = $row[$field];
                    }
                    $query .= ") VALUES (";
                    foreach ($data_set as $data_set_index => $data) {
                        if (is_null($data)) {
                            $query .= 'NULL';
                        } else {
                            $field_type = $fields_type[$data_set_index];
                            if (strpos($field_type, 'BINARY') !== false) {
                                $field_type = 'BINARY';
                            }
                            if (strpos($field_type, 'BLOB') !== false) {
                                $field_type = 'BLOB';
                            }
                            switch ($field_type) {
                            case 'GEOMETRY':
                            case 'POINT':
                            case 'LINESTRING':
                            case 'POLYGON':
                            case 'MULTIPOINT':
                            case 'MULTILINESTRING':
                            case 'MULTIPOLYGON':
                            case 'GEOMETRYCOLLECTION':
                            case 'BLOB':
                            case 'BINARY':
                                $hexEncoded = bin2hex($data);
                                $data       = "x'$hexEncoded'";
                                $query .= $data;
                                break;
                            default:
                                $data = str_replace("\n", "\\n", addslashes($data));
                                $query .= "'{$data}'";
                                break;
                            }
                            // $data = $wpdb->remove_placeholder_escape($data);
                        }
                        if (count($data_set) > $data_set_index + 1) {
                            $query .= ", ";
                        }
                    }
                    $query .= ");";
                    fwrite($fp, $query . PHP_EOL);
                }
                if ($count < 50) {
                    break;
                }
                $offset += 50;
            }
            fwrite($fp, PHP_EOL);
        }
        fclose($fp);
        if (file_exists($sql_full_path)) {
            $zip           = new ZipArchive();
            $zip_file_name = $sql_name . '.zip';
            $zip_file_path = sys_get_temp_dir() . '/' . $zip_file_name;
            $zip->open($zip_file_path, ZipArchive::CREATE | ZipArchive::OVERWRITE);
            if (!$zip) {
                die('ZIP壓縮程式執行錯誤');
            }
            $zip->addFile($sql_full_path, $sql_name);
            // Ref: https://www.php.net/manual/en/zip.constants.php
            // if (method_exists($zip, 'setCompressionIndex')) {
            //     $zip->setCompressionName($sql_name, ZipArchive::CM_DEFLATE);
            // }
            $zip->close();
            if (file_exists($zip_file_path)) {
                header('Content-Description: File Transfer');
                header('Content-Type: application/octet-stream');
                header('Content-Disposition: attachment; filename="' . $zip_file_name . '"');
                header('Expires: 0');
                header('Cache-Control: no-cache');
                header('Pragma: public');
                header('Content-Length: ' . filesize($zip_file_path));
                header('Set-Cookie:fileLoading=true');
                header("Pragma: no-cache");
                readfile($zip_file_path);
                unlink($zip_file_path);
                unlink($sql_full_path);
                exit;
            } else {
                header('Content-Description: File Transfer');
                header('Content-Type: application/octet-stream');
                header('Content-Disposition: attachment; filename="' . $sql_name . '"');
                header('Expires: 0');
                header('Cache-Control: no-cache');
                header('Pragma: public');
                header('Content-Length: ' . filesize($sql_full_path));
                header('Set-Cookie:fileLoading=true');
                header("Pragma: no-cache");
                readfile($sql_full_path);
                unlink($sql_full_path);
                exit;
            }
        } else {
            die('匯出資料庫檔案不存在！');
        }
    }
    public function mxp_ajax_current_plugin_download_action() {

        if (!is_super_admin()) {
            exit('此功能僅限網站最高權限管理人員使用。');
        }
        $path    = sanitize_text_field($_GET['path']);
        $type    = sanitize_text_field($_GET['type']);
        $context = sanitize_text_field($_GET['context']);
        if (empty($path) || empty($type) || !in_array($type, array('file', 'folder'), true) || empty($context) || !wp_verify_nonce($_GET['_wpnonce'], 'mxp-download-current-plugins-' . $path)) {
            exit('驗證請求失敗，請再試一次。');
        }
        $path = base64_decode($path, true);
        if ($path == false) {
            exit('路徑驗證請求失敗，請再試一次。');
        }
        $zip           = new ZipArchive();
        $zip_file_name = '';
        $zip_file_path = '';
        if ($type == 'file') {
            $zip_file_name = basename($path) . '.zip';
            $zip_file_path = sys_get_temp_dir() . '/' . $zip_file_name;
            $zip->open($zip_file_path, ZipArchive::CREATE | ZipArchive::OVERWRITE);
            if (!$zip) {
                exit('ZIP壓縮程式執行錯誤');
            }
            $zip->addFromString(basename($path), file_get_contents($path));
        } else {
            $split_path    = explode(DIRECTORY_SEPARATOR, $path);
            $zip_file_name = $split_path[count($split_path) - 2] . '.zip';
            $relative_path = dirname($path, 2);
            $zip_file_path = sys_get_temp_dir() . '/' . $zip_file_name;
            $zip->open($zip_file_path, ZipArchive::CREATE | ZipArchive::OVERWRITE);
            if (!$zip) {
                exit('ZIP壓縮程式執行錯誤');
            }
            $files = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator(dirname($path)),
                RecursiveIteratorIterator::LEAVES_ONLY
            );
            foreach ($files as $name => $file) {
                if (!$file->isDir()) {
                    $file_path         = $file->getRealPath();
                    $zip_relative_path = str_replace($relative_path . '/', '', $file_path);
                    $zip->addFile($file_path, $zip_relative_path);
                }
            }
        }
        $zip->close();
        if ($zip_file_path != '' && file_exists($zip_file_path)) {
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . $zip_file_name . '"');
            header('Expires: 0');
            header('Cache-Control: no-cache');
            header('Pragma: public');
            header('Content-Length: ' . filesize($zip_file_path));
            header('Set-Cookie:fileLoading=true');
            header("Pragma: no-cache");
            readfile($zip_file_path);
            unlink($zip_file_path);
            exit;
        }
    }

    public function page_wraper($title, $cb) {
        echo '<div class="wrap" id="mxp"><h1>' . $title . '</h1>';
        call_user_func($cb);
        echo '</div>';
    }
    public function build_table($array) {
        // start table
        $html = '<table>';
        // header row
        $html .= '<tr>';
        foreach ($array[0] as $key => $value) {
            $html .= '<th>' . esc_html($key) . '</th>';
        }
        $html .= '</tr>';

        // data rows
        foreach ($array as $key => $value) {
            $html .= '<tr>';
            foreach ($value as $key2 => $value2) {
                $html .= '<td align="center">' . $value2 . '</td>';
            }
            $html .= '</tr>';
        }
        $html .= '</table>';
        $html .= '<style>table, th, td {border: 1px solid black;}</style>';
        return $html;
    }
    public function main_page_cb() {
        $this->page_wraper('開發常用外掛', function () {
            require_once 'includes/class_plugins_list_table.php';

            $plugins_list = new Mxp_Plugins_List_Table();
            $plugins_list->prepare_items();
            $plugins_list->display();
            wp_localize_script($this->plugin_slug . '-plugins-list', 'Mxp_AJAX', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce'   => wp_create_nonce('mxp-ajax-nonce-for-plugin-list'),
            ));

            wp_enqueue_script($this->plugin_slug . '-plugins-list');

        });
    }
    public function mxp_init_author_plugins_table($args) {
        add_filter('install_plugins_table_api_args_mxp-plugins', function ($args) {
            global $paged;
            return [
                'page'     => $paged,
                'per_page' => 100,
                'locale'   => get_user_locale(),
                'author'   => 'mxp',
            ];
        });
        add_filter('install_plugins_nonmenu_tabs', function ($tabs) {
            $tabs[] = 'mxp-plugins';
            return $tabs;
        });
    }

    public function recentlymodfiles_page_cb() {
        $this->page_wraper('查詢最近變更的檔案', function () {
            wp_enqueue_script('jquery-ui-datepicker-google', '//ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/jquery-ui.min.js');
            wp_enqueue_script('jquery-ui-datepicker-google-i18n', '//ajax.googleapis.com/ajax/libs/jqueryui/1.11.2/i18n/jquery-ui-i18n.js');
            wp_enqueue_style('jquery-ui-datepicker-style', '//ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/themes/smoothness/jquery-ui.css');

            wp_localize_script($this->plugin_slug . '-dashboard', 'Mxp_AJAX_dashboard', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce'   => wp_create_nonce('mxp-ajax-nonce-for-recently_mod_files'),
            ));
            wp_enqueue_script($this->plugin_slug . '-dashboard');

            $day_from = date('Y/m/d', strtotime("-1 days"));
            $day_to   = date('Y/m/d', time());
            if (isset($_GET['day_from']) && $_GET['day_from'] != '') {
                $day_from = sanitize_text_field($_GET['day_from']);
            }
            if (isset($_GET['day_to']) && $_GET['day_to'] != '') {
                $day_to = sanitize_text_field($_GET['day_to']);
            }
            echo '<input id="search_day_from" type="text" value="' . esc_attr($day_from) . '" placeholder="查詢起始日"/><input id="search_day_to" type="text" value="' . esc_attr($day_to) . '" placeholder="查詢結束日"/><button type="buttin" class="button button-primary" id="go_search_day">送出</button></br>';
            $raw_data = $this->get_recently_mod_files($day_from, $day_to);
            foreach ($raw_data as $type => $rows) {
                if (count($rows) > 0) {
                    $mod_rows  = array();
                    $type_name = '';
                    switch ($type) {
                    case 'dot_files':
                        $type_name = '作業系統隱藏檔案';
                        break;
                    case 'core_files':
                        $type_name = 'WordPress 核心系統檔案';
                        break;
                    case 'uploads_files':
                        $type_name = '使用者上傳目錄檔案';
                        break;
                    case 'uncategorized_files':
                        $type_name = '未分類檔案';
                        break;
                    case 'theme_files':
                        $type_name = '主題檔案';
                        break;
                    case 'plugin_files':
                        $type_name = '外掛檔案';
                        break;
                    default:
                        break;
                    }
                    if ($type == 'dot_files') {
                        echo '<h3>所有的 <font color="red">' . $type_name . '</font></h3>';
                    } else {
                        echo '<h3>' . $day_from . ' ~ ' . $day_to . ' 內有修改紀錄的 <font color="red">' . $type_name . '</font></h3>';
                    }
                    usort($rows, function ($item1, $item2) {
                        if ($item1['mod_time'] == $item2['mod_time']) {
                            return 0;
                        }
                        return $item1['mod_time'] > $item2['mod_time'] ? -1 : 1;
                    });
                    foreach ($rows as $key => $row) {
                        $full_path     = $row['full_path'];
                        $relative_path = explode(DIRECTORY_SEPARATOR, $row['relative_path']);
                        if (count($relative_path) > 1) {
                            $relative_path[0] = '<strong><font color="orange">' . $relative_path[0] . '</font></strong>';
                            $relative_path    = implode(DIRECTORY_SEPARATOR, $relative_path);
                        } else {
                            $relative_path = $relative_path[0];
                        }
                        $_file = pathinfo($row['name']);

                        $mod_time = date('Y-m-d H:i:s', $row['mod_time']);
                        if (isset($_file['extension']) && in_array(strtoupper($_file['extension']), array('PHP', 'PHP4', 'PHP5', 'CGI', 'PL', 'EXE', 'SH', 'ICO'))) {
                            $name = '<strong><font color="coral">' . esc_html($row['name']) . '</font></strong>';
                        } else {
                            $name = '<font color="blue">' . esc_html($row['name']) . '</font>';
                        }
                        $mxp_download_action_link = admin_url('admin-ajax.php?action=mxp_current_plugin_download&path=' . base64_encode($full_path) . '&type=file&context=recently_mod_file');
                        $mxp_download_action_link = add_query_arg('_wpnonce', wp_create_nonce('mxp-download-current-plugins-' . base64_encode($full_path)), $mxp_download_action_link);
                        $download_link            = '<a target="_blank" href="' . esc_url($mxp_download_action_link) . '" class="mxp_plugin_download_link" class="button">下載</a>';
                        $mod_rows[]               = array(
                            '路徑'   => $relative_path,
                            '檔案名稱' => $name,
                            '修改日期' => $mod_time,
                            '操作'   => $download_link,
                        );
                    }
                    echo $this->build_table($mod_rows);
                }
            }

        });
    }

    public function wpdboptimizemethods_page_cb() {
        $this->page_wraper('WordPress 資料庫最佳化相關功能', function () {
            wp_localize_script($this->plugin_slug . '-db-optimize', 'MXP', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce'   => wp_create_nonce('mxp-ajax-nonce-for-db-optimize'),
            ));
            wp_enqueue_script($this->plugin_slug . '-db-optimize');
            global $wpdb;
            $big_options = $wpdb->get_results("SELECT option_name AS `Option Name`, LENGTH(option_value) AS `Size` FROM {$wpdb->options} WHERE autoload='yes' ORDER BY length(option_value) DESC LIMIT 25", ARRAY_A);
            foreach ($big_options as $key => $option) {
                $option['Size']    = round($option['Size'] / 1024, 2) . ' KB';
                $option['操作']  = '<button type="button" class="autoload_off_btn button button-secondary" data-option_name="' . esc_attr($option['Option Name']) . '">取消 Autoload</button>';
                $big_options[$key] = $option;
            }
            echo '<h3>資料庫排名前 25 筆耗資源紀錄</h3>';
            echo $this->build_table($big_options);
            echo '<h3>清除未使用的主題、外掛設定</h3>';
            echo '<p>許多外掛、主題安裝啟用測試後會在設定資料表中留下紀錄，如果外掛沒有自行清除就會累積，造成網站載入變慢的問題。</p>';
            echo '<button id="go_clean_options" data-step="1" type="button" class="button button-primary">取得清除清單</button></br>';
            echo '<div id="go_clean_options_result"></div></br>';
            echo '<h3>清除孤立的 Post/Comment Meta 資料</h3>';
            echo '<p>不論是手動刪除或是外掛刪除內容，可能沒連帶刪除的 Meta 關聯資料，堆積在資料庫裡變成垃圾。</p>';
            $orphan_postmeta_count    = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->postmeta} pm LEFT JOIN {$wpdb->posts} wp ON wp.ID = pm.post_id WHERE wp.ID IS NULL");
            $orphan_commentmeta_count = $wpdb->get_var("SELECT COUNT(*) as row_count FROM {$wpdb->commentmeta} WHERE comment_id NOT IN (SELECT comment_id FROM {$wpdb->comments})");
            echo '<p><button id="go_clean_orphan_postmeta" type="button" class="button button-primary">清除 ' . $orphan_postmeta_count . ' 筆 Post 孤立資料</button></p>';
            echo '<p><button id="go_clean_orphan_commentmeta" type="button" class="button button-primary">清除 ' . $orphan_commentmeta_count . ' 筆 Comment 孤立資料</button></p>';
            echo '<div id="go_clean_orphan_result"></div></br>';
            echo '<h3>建立資料表索引</h3>';
            echo '請搭配安裝外掛： <a href="https://tw.wordpress.org/plugins/index-wp-mysql-for-speed/" target="_blank">Index WP MySQL For Speed</a> 與 <a href="https://tw.wordpress.org/plugins/index-wp-users-for-speed/" target="_blank">Index WP Users For Speed</a> 使用。';
            echo '<h3>重新安裝 WordPress</h3>';
            echo '<p>砍掉重練，也是一種最佳化的必經之路。<font color="red">注意，這項操作不可逆。請小心使用！</font></p>';
            echo '<p><button id="go_reset_wp" type="button" class="button button-primary">重置網站</button></p>';

        });
    }
    public function dbreplacemethods_page_cb() {
        $this->page_wraper('資料庫關鍵字取代', function () {
            global $wpdb;
            echo '當前資料庫:';
            $dbs        = $wpdb->get_results("SHOW DATABASES", ARRAY_A);
            $current_db = $wpdb->dbname;
            if (isset($_GET['dbname']) && $_GET['dbname'] != '') {
                $current_db = sanitize_text_field($_GET['dbname']);
            }
            wp_localize_script($this->plugin_slug . '-db-search-replace', 'MXP', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce'   => wp_create_nonce('mxp-ajax-nonce-for-db-search-replace-' . $current_db),
            ));
            wp_enqueue_script($this->plugin_slug . '-db-search-replace');
            echo '<select id="select_dbname">';
            $check_db_exists = false;
            foreach ($dbs as $db) {
                if ($db['Database'] != 'information_schema') {
                    echo '<option value="' . esc_attr($db['Database']) . '" ' . selected($current_db, $db['Database']) . '>' . esc_html($db['Database']) . '</option>';
                    if ($db['Database'] == $current_db) {
                        $check_db_exists = true;
                    }
                }
            }
            if (!$check_db_exists) {
                $current_db = $wpdb->dbname;
            }
            echo '</select></br>';
            $tables = $wpdb->get_results("SHOW FULL TABLES FROM `{$current_db}`", ARRAY_A);
            echo '1. 選擇要取代內文的資料表:</br>';
            $tables_arr   = array();
            $tables_arr[] = array('勾選' => '<input type="checkbox" id="check_all" class="check_all" name="check_all" value="ALL">', '資料表' => '全部資料表', '操作' => '');
            echo '<fieldset>';
            foreach ($tables as $key => $table) {
                $table_name = $table['Tables_in_' . $current_db];
                $table_type = $table['Table_type'];
                if ($table_type != 'BASE TABLE') {
                    continue;
                }
                // $break_line = '</br>';
                // echo '<label><input type="checkbox" class="select_tables" name="table[]" value="' . esc_attr($table_name) . '">' . esc_html($table_name) . '<span id="' . $table_name . '"></span></label>' . $break_line;
                $tables_arr[] = array('勾選' => '<input type="checkbox" class="select_tables" name="table[]" value="' . esc_attr($table_name) . '">', '資料表' => esc_html($table_name), '操作' => '<div class="table_result" id="' . esc_attr($table_name) . '"></div>');
            }
            echo $this->build_table($tables_arr);
            echo '</fieldset>';
            echo '2. 設定搜尋的字串(一行一組對應，注意換行前後空白將會被視為搜尋或取代的字元):';
            echo '<fieldset>';
            $demo = 'http://www.mxp.tw' . PHP_EOL . 'http:\/\/www.mxp.tw' . PHP_EOL . 'mxp.tw';
            echo '<textarea id="replace_from" rows="5" cols="50">' . $demo . '</textarea>';
            echo '</fieldset>';
            echo '3. 設定取代的字串(一行一組對應，注意換行前後空白將會被視為搜尋或取代的字元):';
            echo '<fieldset>';
            $demo2 = 'https://www.mxp.tw' . PHP_EOL . 'https:\/\/www.mxp.tw' . PHP_EOL . 'www.mxp.tw';
            echo '<textarea id="replace_to" rows="5" cols="50">' . $demo2 . '</textarea>';
            echo '</fieldset>';
            echo '<fieldset>';
            echo '4. 預覽計算影響的資料筆數';
            echo '</br><button type="button" class="button button-secondary" id="replace_preview">預覽</button></br>';
            echo '<div id="preview_result"></div>';
            echo '</fieldset>';
            echo '<fieldset>';
            echo '5. 確認執行取代';
            echo '</br><button type="button" class="button button-primary" id="go_replace">取代</button></br>';
            echo '<div id="replace_result"></div>';
            echo '</fieldset>';
        });
        // echo $this->build_table($tables);

    }
    public function dbopmethods_page_cb() {
        $this->page_wraper('資料庫檢視與匯出', function () {
            global $wpdb;
            if (isset($_GET['database']) && $_GET['database'] != '') {
                $database_name = sanitize_text_field($_GET['database']);
                $tbs           = $wpdb->get_results(
                    $wpdb->prepare("SELECT TABLE_NAME AS Table_Name, ENGINE AS Engine, TABLE_TYPE AS Table_Type, TABLE_ROWS AS Table_Rows, CREATE_TIME AS Create_Time, TABLE_COLLATION AS Collation, TABLE_COMMENT AS Comment FROM information_schema.TABLES WHERE TABLE_SCHEMA = %s", $database_name), ARRAY_A);
                $wp_tbs        = array_values($wpdb->tables());
                $dropdown_list = array('全部資料表' => '');
                foreach ($tbs as $key => $tabls_info) {
                    $sql_dump_link                           = admin_url('admin-ajax.php?action=mxp_ajax_mysqldump&database=' . $database_name . '&table=' . $tbs[$key]['Table_Name']);
                    $sql_dump_link                           = add_query_arg('_wpnonce', wp_create_nonce('mxp-mysqldump-' . $database_name . '-' . $tbs[$key]['Table_Name']), $sql_dump_link);
                    $download_link                           = '<a target="_blank" href="' . esc_url($sql_dump_link) . '" class="mxp_mysqldump_link button">匯出</a>';
                    $tbs[$key]['操作']                     = $download_link;
                    $dropdown_list[$tbs[$key]['Table_Name']] = $sql_dump_link;
                    if (in_array($tbs[$key]['Table_Name'], $wp_tbs)) {
                        $tbs[$key]['Table_Name'] = '<strong><font color="blue">' . $tbs[$key]['Table_Name'] . '</font></strong>';
                    }
                }
                $sql_dump_link                    = admin_url('admin-ajax.php?action=mxp_ajax_mysqldump&database=' . $database_name . '&table=ALL');
                $sql_dump_link                    = add_query_arg('_wpnonce', wp_create_nonce('mxp-mysqldump-' . $database_name . '-ALL'), $sql_dump_link);
                $dropdown_list['全部資料表'] = $sql_dump_link;
                $table                            = $this->build_table($tbs);
                echo '<a href="' . admin_url("admin.php?page=mxp-db-op-methods") . '">回上一頁</a></br><hr></br>';
                echo '<select id="mxp_dump_select">';
                foreach ($dropdown_list as $table_name => $link) {
                    echo '<option value="' . esc_attr($link) . '">' . esc_html($table_name) . '</option>';
                }
                echo '</select>';
                echo '<button type="button" class="button" id="mxp_dump_btn">匯出</button></br>';
                echo '<script>jQuery("#mxp_dump_btn").click(function(){window.open(jQuery("#mxp_dump_select").val(), "_blank").focus();});</script>';
                echo $table;
            } else {
                $dbs       = $wpdb->get_results("SHOW DATABASES;", ARRAY_A);
                $colls_set = array();
                $colls     = $wpdb->get_results("SHOW COLLATION", ARRAY_A);
                foreach ($colls as $colls_index => $row) {
                    if ($row["Default"]) {
                        $colls_set[$row["Charset"]][-1] = $row["Collation"];
                    } else {
                        $colls_set[$row["Charset"]][] = $row["Collation"];
                    }
                }
                ksort($colls_set);
                foreach ($colls_set as $key => $val) {
                    asort($colls_set[$key]);
                }
                $filter_dbs = [];
                foreach ($dbs as $key => $db) {
                    $database_name = $db['Database'];
                    if ($database_name != 'information_schema') {
                        $collection = $wpdb->get_results(
                            "SHOW CREATE DATABASE {$database_name}"
                            , ARRAY_A);
                        $col        = '';
                        $collection = $collection[0]['Create Database'];
                        if (preg_match('~ COLLATE ([^ ]+)~', $collection, $match)) {
                            $col = $match[1];
                        } elseif (preg_match('~ CHARACTER SET ([^ ]+)~', $collection, $match)) {
                            // default collation
                            $col = $colls_set[$match[1]][-1];
                        }
                        $mod_database_name = $database_name;
                        if ($database_name == $wpdb->dbname) {
                            $mod_database_name = '<strong><font color="blue">
                        ' . $database_name . '</font></strong>';
                        }
                        $filter_dbs[] = array('DB Name' => $mod_database_name, 'Collation' => esc_html($col), '操作' => '<a class="button" href="' . admin_url("admin.php?page=mxp-db-op-methods&database=" . $database_name) . '">查看資料表</a>');
                    }
                }
                $db_server_info = $wpdb->get_results("SHOW VARIABLES like '%version%'", ARRAY_A);
                $table          = $this->build_table($filter_dbs);
                $table2         = $this->build_table($db_server_info);
                echo $table;
                echo '</br>';
                echo $table2;
            }
        });
    }
    public function getwpconfig_page_cb() {
        $this->page_wraper('查看網站各項設定', function () {
            echo '<h2>網路資訊</h2></br>';
            $response = wp_remote_get('https://undo.im/json?from=' . get_site_url());
            if (!is_wp_error($response)) {
                if (200 == wp_remote_retrieve_response_code($response)) {
                    $body = json_decode(wp_remote_retrieve_body($response), true);
                    $ipv4 = '';
                    $ipv6 = '';
                    $ip   = explode('.', $body['IP']);
                    // 找不到 IPv6 的話會噴一個警告，設定這個處理捕捉警告，就不會這麼難看惹 Ref: https://stackoverflow.com/questions/1241728/can-i-try-catch-a-warning
                    set_error_handler(function ($errno, $errstr, $errfile, $errline) {
                        // error was suppressed with the @-operator
                        if (0 === error_reporting()) {
                            return false;
                        }
                        throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
                    });

                    if (count($ip) == 4) {
                        $ipv4 = $body['IP'];
                        $sock = socket_create(AF_INET6, SOCK_DGRAM, SOL_UDP);
                        try {
                            //cloudflare ipv6 dns
                            socket_connect($sock, "2606:4700:4700::1111", 53);
                            socket_getsockname($sock, $qname);
                            socket_close($sock);
                            $ipv6 = $qname;
                        } catch (Exception $ex) {
                            $ipv6 = '無 IPv6 資源！';
                        }
                    } else {
                        $ipv6 = $body['IP'];
                        $sock = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
                        try {
                            socket_connect($sock, "8.8.8.8", 53);
                            socket_getsockname($sock, $qname);
                            socket_close($sock);
                            $ipv4 = $qname;
                        } catch (Exception $ex) {
                            $ipv4 = '無 IPv4 資源！';
                        }
                    }
                    restore_error_handler();
                    $UA             = isset($body['UA']) ? $body['UA'] : '';
                    $asn            = isset($body['CF']['asn']) ? $body['CF']['asn'] : '';
                    $asOrganization = isset($body['CF']['asOrganization']) ? $body['CF']['asOrganization'] : '';
                    $country        = isset($body['CF']['country']) ? $body['CF']['country'] : '';
                    $city           = isset($body['CF']['city']) ? $body['CF']['city'] : '';
                    $timezone       = isset($body['CF']['timezone']) ? $body['CF']['timezone'] : '';
                    // $headers = wp_remote_retrieve_headers( $response );
                    $html = '<table><thead><tr><th colspan="2">當前主機資源</th></tr></thead><tbody>';
                    $html .= '<tr><td><strong>IPv4</strong></td><td>' . $ipv4 . "</td></tr>";
                    $html .= '<tr><td><strong>IPv6</strong></td><td>' . $ipv6 . "</td></tr>";
                    $html .= '<tr><td><strong>User Agent</strong></td><td>' . $UA . "</td></tr>";
                    $html .= '<tr><td><strong>ASN</strong></td><td>' . $asn . "</td></tr>";
                    $html .= '<tr><td><strong>Organization</strong></td><td>' . $asOrganization . "</td></tr>";
                    $html .= '<tr><td><strong>Country</strong></td><td>' . $country . "</td></tr>";
                    $html .= '<tr><td><strong>City</strong></td><td>' . $city . "</td></tr>";
                    $html .= '<tr><td><strong>TimeZone</strong></td><td>' . $timezone . "</td></tr>";
                    $html .= '</tbody></table>';
                    echo $html;
                } else {
                    $error_message = wp_remote_retrieve_response_message($response);
                    echo "查詢異常: " . $error_message . "<br>" . PHP_EOL;
                }
            } else {
                $error_message = $response->get_error_message();
                echo "請求異常: " . $error_message . "<br>" . PHP_EOL;
            }
            echo '<hr></br>';
            highlight_string(file_get_contents(ABSPATH . 'wp-config.php'));
            echo '<hr></br>';
            echo '頁面功能參考外掛：<a href="https://tw.wordpress.org/plugins/system-dashboard/">System Dashboard</a></br>';
            echo $this->build_table($this->get_wp_all_contants());
            echo '<hr></br>';
            echo '<h2>.htaccess 檔案內容</h2></br>';
            if (file_exists(ABSPATH . '.htaccess')) {
                highlight_string(file_get_contents(ABSPATH . '.htaccess'));
            } else {
                echo '檔案不存在。';
            }
            echo '<hr></br>';
            echo '<h2>.user.ini 檔案內容</h2></br>';
            if (file_exists(ABSPATH . '.user.ini')) {
                highlight_string(file_get_contents(ABSPATH . '.user.ini'));
            } else {
                echo '檔案不存在。';
            }
        });
    }
    public function themearchive_page_cb() {
        $this->page_wraper('主題打包工具', function () {
            $all_themes = wp_get_themes();
            $themes_arr = array();
            foreach ($all_themes as $theme_slug => $theme_info) {
                $child_theme = '';
                if ($child_theme = $theme_info->parent()) {
                    $child_theme = $child_theme->display('Name');
                } else {
                    $child_theme = '無';
                }
                $path                     = $theme_info->get_stylesheet_directory() . '/style.css';
                $type                     = 'folder';
                $context                  = 'themes';
                $mxp_download_action_link = admin_url('admin-ajax.php?action=mxp_current_plugin_download&path=' . base64_encode($path) . '&type=' . $type . '&context=' . $context);
                $mxp_download_action_link = add_query_arg('_wpnonce', wp_create_nonce('mxp-download-current-plugins-' . base64_encode($path)), $mxp_download_action_link);
                $download_link            = '<a class="button" target="_blank" href="' . esc_url($mxp_download_action_link) . '" class="mxp_plugin_download_link">打包主題</a>';
                // echo "<li>" . $child_theme . " " . $theme_info->display('Name') . "(" . $theme_info->display('Version') . ") ->" . $theme_info->display('Status') . " By " . $theme_info->display('Author') . " | " . $download_link . "</li>";
                $themes_arr[] = array('名稱' => $theme_info->display('Name'), '上層主題' => $child_theme, '作者' => $theme_info->display('Author'), '操作' => $download_link);
            }
            echo $this->build_table($themes_arr);
        });
    }
    public function searchplugin_page_cb() {
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        $all_plugins = get_plugins();
        $slugs       = array();
        foreach ($all_plugins as $key => $info) {
            $slug    = explode('/', $key);
            $slugs[] = $slug[0];
        }
        wp_localize_script($this->plugin_slug . '-search-plugins', 'MXP', array(
            'ajaxurl'         => admin_url('admin-ajax.php'),
            'nonce'           => wp_create_nonce('mxp-ajax-nonce-for-search-plugins'),
            'install_plugins' => $slugs,
        ));

        wp_enqueue_script($this->plugin_slug . '-search-plugins');
        echo '<div id="search_iframe"></div>';
    }

    public function listauthorplugin_page_cb() {
        require_once ABSPATH . 'wp-admin/includes/class-wp-plugin-install-list-table.php';
        $transient = 'mxp-plugins';
        $cached    = get_transient($transient);
        if (false !== $cached) {
            echo $cached;
            return;
        }
        $_POST['tab'] = 'mxp-plugins';
        ob_start();
        $table = new WP_Plugin_Install_List_Table();
        $table->prepare_items();
        $table->display();
        $content = ob_get_clean();
        set_transient($transient, $content, DAY_IN_SECONDS);
        echo $content;
    }
    public function changepostowner_page_cb() {
        $this->page_wraper('修改全站內容權限', function () {
            $ps     = get_post_types(array('exclude_from_search' => false), 'names', 'or');
            $select = '<p>選擇內容類型： <select name="mxp_dev_post_type"><option value="">All</option>';
            foreach ($ps as $key => $value) {
                $select .= '<option value="' . $value . '">' . $value . '</option>';
            }
            $select .= '</select></p>';
            echo "<form name='post_owner' method='POST'>" . $select . "<p>選擇使用者：" . wp_dropdown_users(array('name' => 'mxp_dev_post_author', 'selected' => 1, 'echo' => false)) . "</p><input type='hidden' name='nonce' value='" . wp_create_nonce('mxp-dev-change-owner-page') . "'><input type='submit' value='送出' class='button action'/></form>";
        });
        if (isset($_POST['nonce']) && wp_verify_nonce($_POST['nonce'], 'mxp-dev-change-owner-page') && isset($_POST['mxp_dev_post_author'])) {
            global $wpdb;
            $uid  = 1;
            $type = empty($_POST['mxp_dev_post_type']) ? "" : $_POST['mxp_dev_post_type'];
            if (is_numeric($_POST['mxp_dev_post_author'])) {
                $uid = $_POST['mxp_dev_post_author'];
            }
            if ($type == "") {
                $wpdb->query($wpdb->prepare("UPDATE {$wpdb->prefix}posts SET post_author = %s", $uid));
            } else {
                $wpdb->query($wpdb->prepare("UPDATE {$wpdb->prefix}posts SET post_author = %s WHERE post_type = %s", $uid, $type));
            }
            echo "<p>更新成功！</p>";
        } else if (!empty($_POST)) {
            echo "<p>錯誤的操作！</p>";
        }
    }
    public function themeforest_page_cb() {

        $this->page_wraper('Themeforest <a href="https://goo.gl/Oh9cK5" target="_blank">授權碼</a>', function () {
            echo '<form name="themeforest" method="get"><input type="hidden" name="page" value="mxp-themeforest-list"><input type="text" name="code" value="" size="40"/><input type="submit" value="送出" class="button action"/></form>';
        });

        if (isset($_GET['code']) && !empty($_GET['code'])) {
            $auth_code = sanitize_text_field($_GET['code']);
            $response  = '';
            $mxp_flag  = false;
            if (strpos($auth_code, 'MXP_') === 0) {
                $mxp_flag = true;
                $response = wp_remote_get('https://api.undo.im/wp-json/mxp_knockers/v1/app/list-purchases', array(
                    'headers' => array('Authorization' => 'Bearer ' . $auth_code),
                ));
            } else {
                $response = wp_remote_get($this->themeforest_api_base_url . '/market/buyer/list-purchases', array(
                    'headers' => array('Authorization' => 'Bearer ' . $auth_code),
                ));
            }
            if (is_array($response) && !is_wp_error($response) && $response['response']['code'] == 200) {
                $resp = json_decode($response['body'], true);
                if ($resp && !isset($resp['error']) && $resp['count'] != '0') {
                    $datas   = $resp['results'];
                    $themes  = [];
                    $plugins = [];
                    $others  = [];
                    for ($i = 0; $i < count($datas); ++$i) {
                        $dlinks = array();
                        if (!$mxp_flag) {
                            $dobj = wp_remote_get($this->themeforest_api_base_url . '/market/buyer/download?item_id=' . $datas[$i]['item']['id'], array(
                                'headers' => array('Authorization' => 'Bearer ' . $auth_code),
                            ));
                            if (is_array($dobj) && !is_wp_error($dobj) && $dobj['response']['code'] == 200) {
                                $dlinks = json_decode($dobj['body'], true);
                                if (!$dlinks) {
                                    echo '<br/>發生錯誤，請回報下列錯誤資訊至：im@mxp.tw<br/><br/><pre>' . esc_html(print_r($dobj, true)) . '</pre>';
                                    wp_die();
                                }
                            } else {
                                echo '<br/>發生錯誤，請回報下列錯誤資訊至：im@mxp.tw<br/><br/><pre>' . esc_html(print_r($dobj, true)) . '</pre>';
                                wp_die();
                            }
                        } else {
                            $dlinks['wordpress_plugin'] = $datas[$i]['item']['wordpress_plugin_metadata']['dlink'];
                        }
                        if (isset($datas[$i]['item']['wordpress_theme_metadata'])) {
                            $tid                                                    = $datas[$i]['item']['id'];
                            $datas[$i]['item']['wordpress_theme_metadata']['id']    = esc_attr($tid);
                            $datas[$i]['item']['wordpress_theme_metadata']['dlink'] = esc_url($dlinks['wordpress_theme']);
                            $datas[$i]['item']['wordpress_theme_metadata']['code']  = $datas[$i]['code'];
                            $themes[]                                               = $datas[$i]['item']['wordpress_theme_metadata'];
                        } else if (isset($datas[$i]['item']['wordpress_plugin_metadata'])) {
                            $tid                                                     = $datas[$i]['item']['id'];
                            $datas[$i]['item']['wordpress_plugin_metadata']['id']    = esc_attr($tid);
                            $datas[$i]['item']['wordpress_plugin_metadata']['dlink'] = esc_url($dlinks['wordpress_plugin']);
                            $datas[$i]['item']['wordpress_plugin_metadata']['code']  = $datas[$i]['code'];
                            $plugins[]                                               = $datas[$i]['item']['wordpress_plugin_metadata'];

                        } else {
                            $datas[$i]['item']['dlink'] = esc_url($dlinks['download_url']);
                            $datas[$i]['item']['code']  = $datas[$i]['code'];
                            $others[]                   = $datas[$i]['item'];
                        }
                    } //end for-loop
                    echo '<h1>主題</h1><br/><table style="text-align:center;"><tr><th>操作</th><th>名稱</th><th>版本</th><th>購買序號</th></tr>';
                    for ($i = 0; $i < count($themes); ++$i) {
                        echo "<tr><td><button class='install_theme' data-dlink='{$themes[$i]['dlink']}' data-id='{$themes[$i]['id']}'>下載＆安裝</button><button style='display:none;' class='activate_theme' data-id='{$themes[$i]['id']}'>前往主題頁啟動</button></td><td>" . esc_html($themes[$i]['theme_name']) . "</td><td>" . esc_html($themes[$i]['version']) . "</td><td>" . esc_html($themes[$i]['code']) . "</td>";
                    }
                    echo '</table>';
                    echo '<h1>外掛</h1><br/><table style="text-align:center;"><tr><th>操作</th><th>名稱</th><th>版本</th><th>購買序號</th></tr>';
                    for ($i = 0; $i < count($plugins); ++$i) {
                        $pname = esc_attr($plugins[$i]['plugin_name']);
                        echo "<tr><td><button class='install_plugin' data-name='{$pname}' data-dlink='{$plugins[$i]['dlink']}' data-id='{$plugins[$i]['id']}'>下載＆安裝</button><button style='display:none;' class='activate_plugin' data-name='{$pname}' data-dlink='{$plugins[$i]['dlink']}' data-id='{$plugins[$i]['id']}'>啟動</button></td><td>" . esc_html($plugins[$i]['plugin_name']) . "</td><td>" . esc_html($plugins[$i]['version']) . "</td><td>" . esc_html($plugins[$i]['code']) . "</td>";
                    }
                    echo '</table>';
                    echo '<h1>其他（未分類）</h1><br/><table style="text-align:center;"><tr><th>操作</th><th>名稱</th><th>版本</th><th>購買序號</th></tr>';
                    for ($i = 0; $i < count($others); ++$i) {
                        $oname = esc_html($others[$i]['name']);
                        echo "<tr><td><button class='install_other' data-dlink='{$others[$i]['dlink']}' data-id='{$others[$i]['id']}'>下載手動安裝</button></td><td>{$oname}</td><td>NONE</td><td>" . $others[$i]['code'] . "</td>";
                    }
                    echo '</table>';
                    wp_localize_script($this->plugin_slug . '-themeforest-list', 'Mxp_AJAX', array(
                        'ajaxurl'   => admin_url('admin-ajax.php'),
                        'themesurl' => admin_url('themes.php'),
                        'nonce'     => wp_create_nonce('mxp-ajax-nonce-for-themeforest-list'),
                    ));
                    wp_enqueue_script($this->plugin_slug . '-themeforest-list');
                } else {
                    echo '<br/>若非無購買項目，請回報下列錯誤資訊至：im@mxp.tw<br/><br/><pre>' . print_r($response, true) . '</pre>';
                    wp_die();
                }
            } else {
                echo '<br/>若非授權碼錯誤或無購買項目，請回報下列錯誤資訊至：im@mxp.tw<br/><br/><pre>' . print_r($response, true) . '</pre>';
                wp_die();
            }
        } else {
            echo '<p>請輸入授權碼！</p>';
        }
    }
    public function mxp_ajax_install_plugin() {
        $nonce = sanitize_text_field(isset($_POST['nonce']) == true ? $_POST['nonce'] : "");
        if (!wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-plugin-list') && !wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-themeforest-list')) {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '錯誤的請求來源')));
        }
        $activated = sanitize_text_field(isset($_POST['activated']) == true ? $_POST['activated'] : "");
        $file      = sanitize_text_field(isset($_POST['file']) == true ? $_POST['file'] : "");
        $dlink     = isset($_POST['dlink']) == true ? $_POST['dlink'] : "";
        $slug      = sanitize_text_field(isset($_POST['slug']) == true ? $_POST['slug'] : "");
        $version   = sanitize_text_field(isset($_POST['version']) == true ? $_POST['version'] : "");
        $name      = sanitize_text_field(isset($_POST['name']) == true ? $_POST['name'] : "");
        if ($activated == "" || $dlink == "" || $slug == "" || $version == "" || $name == "") {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '錯誤的請求資料')));
        }
        if ($activated === 'true' || $file != 'false') {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '已經安裝')));
        }
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        //code reference from wp-admin/includes/ajax-actions.php
        $skin     = new WP_Ajax_Upgrader_Skin();
        $upgrader = new Plugin_Upgrader($skin);
        $result   = $upgrader->install($dlink);
        if (defined('WP_DEBUG') && WP_DEBUG) {
            $status['debug'] = $skin->get_upgrade_messages();
        }
        if (is_wp_error($result)) {
            $status['errorCode']    = $result->get_error_code();
            $status['errorMessage'] = $result->get_error_message();
            wp_send_json_error($status);
        } elseif (is_wp_error($skin->result)) {
            $status['errorCode']    = $skin->result->get_error_code();
            $status['errorMessage'] = $skin->result->get_error_message();
            wp_send_json_error($status);
        } elseif ($skin->get_errors()->get_error_code()) {
            $status['errorMessage'] = $skin->get_error_messages();
            wp_send_json_error($status);
        } elseif (is_null($result)) {
            global $wp_filesystem;
            $status['errorCode']    = 'unable_to_connect_to_filesystem';
            $status['errorMessage'] = __('Unable to connect to the filesystem. Please confirm your credentials.');
            // Pass through the error from WP_Filesystem if one was raised.
            if ($wp_filesystem instanceof WP_Filesystem_Base && is_wp_error($wp_filesystem->errors) && $wp_filesystem->errors->get_error_code()) {
                $status['errorMessage'] = esc_html($wp_filesystem->errors->get_error_message());
            }
            wp_send_json_error($status);
        }
        if (!function_exists('install_plugin_install_status')) {
            require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
        }
        $pluginInfo     = install_plugin_install_status(array('name' => $name, 'slug' => $slug, 'version' => $version));
        $status['info'] = json_encode($pluginInfo);
        wp_send_json_success($status);
    }
    public function mxp_ajax_install_plugin_from_url() {
        $nonce = sanitize_text_field(isset($_POST['nonce']) == true ? $_POST['nonce'] : "");
        if (!wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-search-plugins')) {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '錯誤的請求來源')));
        }

        $dlink = isset($_POST['dlink']) == true ? $_POST['dlink'] : "";
        if ($dlink == "") {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '錯誤，無帶入下載連結')));
        }
        $slug = sanitize_text_field(isset($_POST['slug']) == true ? $_POST['slug'] : "");
        $name = sanitize_text_field(isset($_POST['name']) == true ? $_POST['name'] : "");
        if ($slug == "" || $name == "") {
            wp_send_json_error(array('status' => false, 'data' => array('msg' => '錯誤的外掛資料')));
        }
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        //code reference from wp-admin/includes/ajax-actions.php
        $skin     = new WP_Ajax_Upgrader_Skin();
        $upgrader = new Plugin_Upgrader($skin);
        $result   = $upgrader->install($dlink);
        if (defined('WP_DEBUG') && WP_DEBUG) {
            $status['debug'] = $skin->get_upgrade_messages();
        }
        if (is_wp_error($result)) {
            $status['errorCode']    = $result->get_error_code();
            $status['errorMessage'] = $result->get_error_message();
            wp_send_json_error($status);
        } elseif (is_wp_error($skin->result)) {
            $status['errorCode']    = $skin->result->get_error_code();
            $status['errorMessage'] = $skin->result->get_error_message();
            wp_send_json_error($status);
        } elseif ($skin->get_errors()->get_error_code()) {
            $status['errorMessage'] = $skin->get_error_messages();
            wp_send_json_error($status);
        } elseif (is_null($result)) {
            global $wp_filesystem;
            $status['errorCode']    = 'unable_to_connect_to_filesystem';
            $status['errorMessage'] = __('Unable to connect to the filesystem. Please confirm your credentials.');
            // Pass through the error from WP_Filesystem if one was raised.
            if ($wp_filesystem instanceof WP_Filesystem_Base && is_wp_error($wp_filesystem->errors) && $wp_filesystem->errors->get_error_code()) {
                $status['errorMessage'] = esc_html($wp_filesystem->errors->get_error_message());
            }
            wp_send_json_error($status);
        }
        if (!function_exists('install_plugin_install_status')) {
            require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
        }
        $pluginInfo     = install_plugin_install_status(array('name' => $name, 'slug' => $slug, 'version' => '0.1'));
        $status['info'] = $pluginInfo;
        wp_send_json_success($status);
    }
    public function mxp_ajax_activate_plugin() {
        $nonce = sanitize_text_field(isset($_POST['nonce']) == true ? $_POST['nonce'] : "");
        if (!wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-plugin-list') && !wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-themeforest-list')) {
            wp_send_json_error(array('activated' => false, 'data' => array('msg' => '錯誤的請求')));
        }
        $name = sanitize_text_field(isset($_POST['name']) == true ? $_POST['name'] : "");
        $file = isset($_POST['file']) ? sanitize_text_field(isset($_POST['file']) == true ? $_POST['file'] : "") : $this->get_plugin_file($name);
        if (!isset($file)) {
            wp_send_json_error(array('activated' => false, 'data' => array('msg' => '找不到啟動來源')));
        }
        activate_plugins($file);
        if (is_plugin_active($file)) {
            wp_send_json_success(array('activated' => true));
        } else {
            wp_send_json_error(array('activated' => false));
        }
    }
    public function mxp_ajax_install_theme() {
        $nonce = sanitize_text_field(isset($_POST['nonce']) == true ? $_POST['nonce'] : "");
        $dlink = isset($_POST['dlink']) == true ? $_POST['dlink'] : "";
        if (!wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-themeforest-list') || $dlink == "") {
            wp_send_json_error(array('data' => array('msg' => '錯誤的請求')));
        }
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        //code reference from wp-admin/includes/ajax-actions.php
        $skin     = new WP_Ajax_Upgrader_Skin();
        $upgrader = new Theme_Upgrader($skin);
        $result   = $upgrader->install($dlink);
        if (defined('WP_DEBUG') && WP_DEBUG) {
            $status['debug'] = $skin->get_upgrade_messages();
        }
        if (is_wp_error($result)) {
            $status['errorCode']    = $result->get_error_code();
            $status['errorMessage'] = $result->get_error_message();
            wp_send_json_error($status);
        } elseif (is_wp_error($skin->result)) {
            $status['errorCode']    = $skin->result->get_error_code();
            $status['errorMessage'] = $skin->result->get_error_message();
            wp_send_json_error($status);
        } elseif ($skin->get_errors()->get_error_code()) {
            $status['errorMessage'] = $skin->get_error_messages();
            wp_send_json_error($status);
        } elseif (is_null($result)) {
            global $wp_filesystem;
            $status['errorCode']    = 'unable_to_connect_to_filesystem';
            $status['errorMessage'] = __('Unable to connect to the filesystem. Please confirm your credentials.');
            // Pass through the error from WP_Filesystem if one was raised.
            if ($wp_filesystem instanceof WP_Filesystem_Base && is_wp_error($wp_filesystem->errors) && $wp_filesystem->errors->get_error_code()) {
                $status['errorMessage'] = esc_html($wp_filesystem->errors->get_error_message());
            }
            wp_send_json_error($status);
        }
        wp_send_json_success($status);
    }
    // public function mxp_ajax_dismiss_notice() {
    //     $nonce = sanitize_text_field(isset($_POST['nonce']) == true ? $_POST['nonce'] : "");
    //     $key   = sanitize_text_field(isset($_POST['key']) == true ? $_POST['key'] : "");
    //     if (!wp_verify_nonce($nonce, 'mxp-ajax-nonce-for-dashboard') || $key == "") {
    //         wp_send_json_error(array('data' => array('msg' => '錯誤的請求')));
    //     }
    //     setcookie($this->plugin_slug . '-dissmis-notice-' . $key, '1');
    //     wp_send_json_success('done');
    // }
    public function get_wp_all_contants() {
        // $wp_constants 變數資料量太大，放另一個檔案存
        include dirname(__FILE__) . '/wp_constants.php';
        $output  = array();
        $pre_set = array();
        foreach ($wp_constants as $category) {
            $category_name = $category['title'];
            foreach ($category['constants'] as $constant) {
                if ((defined($constant['name'])) && (!empty(constant($constant['name'])))) {
                    $constant_name                 = $constant['name'];
                    $pre_set[]                     = $constant_name;
                    $constant_default_value        = $constant['value'];
                    $constant_current_value        = constant($constant_name);
                    $constant_current_output_value = '';
                    $constant_description          = $constant['description'];
                    switch (gettype($constant_current_value)) {
                    case 'array':
                    case 'object':
                        $constant_current_output_value = '<pre>' . var_export($constant_current_value, true) . '</pre>';
                        break;
                    case 'boolean':
                        $constant_current_output_value = true === $constant_current_value ? 'true' : 'false';
                        break;
                    case 'NULL':
                        $constant_current_output_value = 'NULL';
                        break;
                    default:
                        $constant_current_output_value = $constant_current_value . '';
                        break;
                    }

                    if ($constant_default_value != $constant_current_value) {
                        $output[] = array('分類' => $category_name, '常數命名' => $constant_name, '說明' => $constant_description, '預設值' => $constant_default_value, '當前值' => '<font color="red">' . $constant_current_output_value . '</font>');
                    } else {
                        $output[] = array('分類' => $category_name, '常數命名' => $constant_name, '說明' => $constant_description, '預設值' => $constant_default_value, '當前值' => $constant_current_output_value);
                    }

                }

            }

        }
        $all_constants  = get_defined_constants(true);
        $user_constants = $all_constants['user'];
        foreach ($user_constants as $constant_name => $value) {
            if (!in_array($constant_name, $pre_set, true)) {
                $category_name                 = '主題/外掛';
                $constant_current_value        = constant($constant_name);
                $constant_current_output_value = '';
                $constant_description          = '自定義常數';
                $constant_default_value        = '';
                switch (gettype($constant_current_value)) {
                case 'array':
                case 'object':
                    $constant_current_output_value = '<pre>' . var_export($constant_current_value, true) . '</pre>';
                    break;
                case 'boolean':
                    $constant_current_output_value = true === $constant_current_value ? 'true' : 'false';
                    break;
                case 'NULL':
                    $constant_current_output_value = 'NULL';
                    break;
                default:
                    $constant_current_output_value = $constant_current_value . '';
                    break;
                }
                $output[] = array('分類' => $category_name, '常數命名' => $constant_name, '說明' => $constant_description, '預設值' => $constant_default_value, '當前值' => $constant_current_output_value);
            }
        }
        return $output;
    }
    public function get_plugin_file($plugin_name) {
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }
        $plugins = get_plugins();
        foreach ($plugins as $plugin_file => $plugin_info) {
            if ($plugin_info['Name'] == $plugin_name) {
                return $plugin_file;
            }
        }
        return null;
    }

}

add_action('plugins_loaded', array('MxpDevTools', 'get_instance'));